@@ -3,7 +3,7 @@
 #
 #  signed/vector valued measures with atomic and diffuse components
 #
-#  $Revision: 1.62 $  $Date: 2017/01/07 02:50:14 $
+#  $Revision: 1.66 $  $Date: 2017/07/10 10:02:34 $
 #
 msr <- function(qscheme, discrete, density, check=TRUE) {
   if(!inherits(qscheme, "quad"))
@@ -267,7 +267,8 @@ plot.msr <- function(x, ..., add=FALSE,
                      how=c("image", "contour", "imagecontour"),
                      main=NULL, 
                      do.plot=TRUE,
-                     multiplot=TRUE) {
+                     multiplot=TRUE,
+                     massthresh=0) {
   if(is.null(main)) 
     main <- short.deparse(substitute(x))
   how <- match.arg(how)
@@ -321,7 +322,13 @@ plot.msr <- function(x, ..., add=FALSE,
   }
   ## scalar measure
   x <- y[[1]]
+  ## get atoms
   xatomic <- (x$loc %mark% x$discrete)[x$atoms]
+  if(length(massthresh) && all(is.finite(massthresh))) {
+    ## ignore atoms with absolute mass <= massthresh
+    check.1.real(massthresh)
+    xatomic <- xatomic[abs(marks(xatomic)) > massthresh]
+  }
   xtra.im <- graphicsPars("image")
   xtra.pp <- setdiff(graphicsPars("ppp"), c("box", "col"))
   xtra.ow <- graphicsPars("owin")
@@ -500,11 +507,19 @@ Ops.msr <- function(e1,e2=NULL){
         stop(paste("Operation", sQuote(paste0("A", .Generic, "B")),
                    "is undefined for measures A, B"),
              call.=FALSE)
-      if(!identical(e1$loc, e2$loc))
-        stop(paste("Cannot perform operation",
-                   paste0(sQuote(.Generic), ":"),
-                   "measures are defined on different quadrature schemes"),
+      k1 <- dim(e1)[2]
+      k2 <- dim(e2)[2]
+      if(k1 != k2) 
+        stop(paste("Operation", sQuote(paste0("A", .Generic, "B")),
+                   "is undefined because A, B have incompatible dimensions:",
+                   "A is", ngettext(k1, "scalar", paste0(k1, "-vector")),
+                   ", B is", ngettext(k2, "scalar", paste0(k2, "-vector"))),
              call.=FALSE)
+      if(!identical(e1$loc, e2$loc)) {
+        haha <- harmonise(e1, e2)
+        e1 <- haha[[1L]]
+        e2 <- haha[[2L]]
+      }
       e1 <- unclass(e1)
       e2 <- unclass(e2)
       e1[vn] <- mapply(.Generic, e1[vn], e2[vn],
@@ -570,3 +585,92 @@ measureVariation <- function(x) {
 
 totalVariation <- function(x) integral(measureVariation(x))
   
+harmonise.msr <- local({
+
+  harmonise.msr <- function(...) {
+    argz <- list(...)
+    n <- length(argz)
+    if(n == 0) return(argz)
+    ismeasure <- sapply(argz, inherits, what="msr")
+    if(!any(ismeasure))
+      stop("No measures supplied")
+    if(!all(ismeasure))
+    stop("All arguments should be measures (objects of class msr)")
+    if(n < 2) return(argz)
+    result <- vector(mode="list", length=n)
+    ## extract entries
+    loclist <- lapply(argz, getElement, name="loc")
+    atomlist <- lapply(argz, getElement, name="atoms")
+    masslist <- lapply(argz, getElement, name="discrete")
+    denslist <- lapply(argz, getElement, name="density")
+    ## check for compatible dimensions of measure values
+    dimen <- unique(sapply(argz, ncol))
+    if(length(dimen) > 1)
+      stop("Measures have different dimensions:", commasep(sort(dimen)))
+    ## check for marked points
+    ismarked <- sapply(loclist, is.marked)
+    if(any(ismarked) && !all(ismarked))
+      stop("Some, but not all, quadrature schemes are marked")
+    ismarked <- all(ismarked)
+    ## union of all quadrature points in all measures
+    Uloc <- do.call(superimpose, append(unname(loclist), list(check=FALSE)))
+    Uloc <- unique(Uloc)
+    nU <- npoints(Uloc)
+    ## match each quadrature set to the union
+    ## and find nearest data point to each point in the union
+    if(!ismarked) {
+      matchlist <- lapply(loclist, nncross, Y=Uloc, what="which")
+      nearlist  <- lapply(loclist, ssorcnn, xx=Uloc, what="which")
+    } else {
+      stop("Not yet implemented for marked quadrature schemes")
+    }
+    ## nearest neighbour interpolation of density values of each argument
+    ## onto the common quadrature set
+    Udenslist <- mapply(extract, x=denslist, i=nearlist,
+                        SIMPLIFY=FALSE)
+    ## initialise other bits
+    noatoms  <- logical(nU) 
+    zeromass <- if(dimen == 1) numeric(nU) else matrix(0, nU, dimen)
+    Uatomlist <- rep(list(noatoms), n)  
+    Umasslist <- rep(list(zeromass), n)
+    ## assign atoms in each argument
+    Uatomlist <- mapply(subsetgets, x=Uatomlist, i=matchlist, value=atomlist,
+                        SIMPLIFY=FALSE)
+    Umasslist <- mapply(subsetgets, x=Umasslist, i=matchlist, value=masslist,
+                        SIMPLIFY=FALSE)
+    ## union of atoms
+    isatom <- Reduce("|", Uatomlist)
+    ## masses at atoms
+    Umasslist <- lapply(Umasslist, extract, i=isatom)
+    ## make common quadrature scheme
+    UQ <- quadscheme(Uloc[isatom], Uloc[!isatom])
+    ## reorder density data correspondingly
+    neworder <- c(which(isatom), which(!isatom))
+    Udenslist <- lapply(Udenslist, extract, i=neworder)
+    ## make new measures
+    result <- mapply(msr,
+                     MoreArgs=list(qscheme=UQ),
+                     discrete=Umasslist,
+                     density=Udenslist,
+                     SIMPLIFY=FALSE)
+    names(result) <- names(argz)
+    class(result) <- unique(c("solist", class(result)))
+    return(result)
+  }
+
+  ssorcnn <- function(xx, yy, what) nncross(xx, yy, what=what)
+  
+  extract <- function(x, i) {
+    if(is.matrix(x)) x[i, , drop=FALSE] else x[i]
+  }
+  subsetgets <- function(x, i, value) {
+    if(is.matrix(x)) {
+      x[i, ] <- value
+    } else {
+      x[i] <- value
+    }
+    return(x)
+  }
+
+  harmonise.msr
+})
