@@ -1,7 +1,7 @@
 #
 # profilepl.R
 #
-#  $Revision: 1.42 $  $Date: 2016/04/25 02:34:40 $
+#  $Revision: 1.43 $  $Date: 2017/06/05 10:31:58 $
 #
 #  computes profile log pseudolikelihood
 #
@@ -228,6 +228,9 @@ as.ppm.profilepl <- function(object) {
   object$fit
 }
 
+predict.profilepl <- function(object, ...) {
+  predict(object$fit, ...)
+}
 
 ##
 ##  plot method 
